import {
  useSupported
} from "./chunk-T2UAVNAT.mjs";
import {
  __name
} from "./chunk-Q3KE6ACF.mjs";

// src/use-media-query.ts
import { defaultWindow, resolveRef, tryOnScopeDispose } from "@chakra-ui/vue-utils";
import { ref, watchEffect } from "vue";
function useMediaQuery(query, options = {}) {
  const {
    window = defaultWindow
  } = options;
  const isSupported = useSupported(() => window && "matchMedia" in window && typeof window.matchMedia === "function");
  let mediaQuery;
  const matches = ref(false);
  const cleanup = /* @__PURE__ */ __name(() => {
    if (!mediaQuery)
      return;
    if ("removeEventListener" in mediaQuery)
      mediaQuery.removeEventListener("change", update);
    else
      mediaQuery.removeListener(update);
  }, "cleanup");
  const update = /* @__PURE__ */ __name(() => {
    if (!isSupported.value)
      return;
    cleanup();
    mediaQuery = window.matchMedia(resolveRef(query).value);
    matches.value = mediaQuery.matches;
    if ("addEventListener" in mediaQuery)
      mediaQuery.addEventListener("change", update);
    else
      mediaQuery.addListener(update);
  }, "update");
  watchEffect(update);
  tryOnScopeDispose(() => cleanup());
  return matches;
}
__name(useMediaQuery, "useMediaQuery");

export {
  useMediaQuery
};
