import {
  __name,
  __publicField
} from "./chunk-Q3KE6ACF.mjs";

// src/use-descendant.ts
import { createContext } from "@chakra-ui/vue-utils";
import { customRef, triggerRef } from "vue";
var DescendantAuthority = class {
  constructor(id) {
    __publicField(this, "descendants");
    __publicField(this, "id");
    this.descendants = [];
    this.id = id;
  }
  register(descendant) {
    this.descendants.push(descendant);
    return descendant.id;
  }
  unregister(id) {
    this.descendants = this.descendants.filter((descendant) => descendant.id !== id);
  }
  isRegistered(id) {
    return this.descendants.some((descendant) => descendant.id === id);
  }
  getDescendants() {
    return this.descendants;
  }
  getDescendantById(id) {
    return this.descendants.find((descendant) => descendant.id === id);
  }
  json() {
    const payload = {
      id: this.id,
      descendants: this.descendants.map((descendant) => ({
        id: descendant.id,
        authorityId: descendant.authorityId
      }))
    };
    return payload;
  }
};
__name(DescendantAuthority, "DescendantAuthority");
var descendantManagerStore = /* @__PURE__ */ new Map();
var Descendant = class {
  constructor({
    id,
    authority
  }) {
    __publicField(this, "id");
    __publicField(this, "authorityId");
    this.id = id;
    this.authorityId = authority.id;
  }
};
__name(Descendant, "Descendant");
function createDescendantAuthority(id) {
  const descendantManager = new DescendantAuthority(id);
  descendantManagerStore.set(id, descendantManager);
  return descendantManager;
}
__name(createDescendantAuthority, "createDescendantAuthority");
function createDescendant(id, authority) {
  return new Descendant({
    id,
    authority
  });
}
__name(createDescendant, "createDescendant");
function storeDescendant(descendant) {
  const {
    id,
    authorityId
  } = descendant;
  const authority = descendantManagerStore.get(authorityId);
  if (authority.isRegistered(id)) {
    throw new Error(`Descendant with id ${id} is already registered.`);
  }
  return authority.register(descendant);
}
__name(storeDescendant, "storeDescendant");
function createDescendantRegister(id) {
  let __descendantAuthority__ = createDescendantAuthority(Symbol(id));
  const descendantAuthority = customRef((track, trigger) => ({
    get() {
      track();
      return __descendantAuthority__;
    },
    set(value) {
      __descendantAuthority__ = value;
      trigger();
    }
  }));
  const [DescendantAuthorityProvider, useAuthority] = createContext({
    name: id,
    strict: false
  }, __descendantAuthority__);
  function registerAuthority() {
    DescendantAuthorityProvider(__descendantAuthority__);
  }
  __name(registerAuthority, "registerAuthority");
  function registerDescendant(id2) {
    const descendant = createDescendant(id2, __descendantAuthority__);
    storeDescendant(descendant);
    triggerRef(descendantAuthority);
    return descendant;
  }
  __name(registerDescendant, "registerDescendant");
  return [registerAuthority, registerDescendant, useAuthority, descendantAuthority];
}
__name(createDescendantRegister, "createDescendantRegister");

export {
  DescendantAuthority,
  Descendant,
  createDescendantRegister
};
