import {
  __name
} from "./chunk-Q3KE6ACF.mjs";

// src/use-counter.ts
import { clampValue, countDecimalPlaces, toPrecision } from "@chakra-ui/utils";
import { ref, watchEffect } from "vue";
function useCounter(props = {}) {
  const {
    precision: precisionProp,
    defaultValue = 0,
    step: stepProp = 1,
    min = Number.MIN_SAFE_INTEGER,
    max = Number.MAX_SAFE_INTEGER,
    keepWithinRange = true
  } = props;
  const valueState = ref((() => {
    var _a;
    if (defaultValue == null)
      return "";
    return (_a = cast(defaultValue, stepProp, precisionProp)) != null ? _a : "";
  })());
  const decimalPlaces = ref(0);
  let precision;
  const update = /* @__PURE__ */ __name((next) => {
    if (next === valueState.value)
      return;
    valueState.value = toPrecision(parse(next), precision).toString();
  }, "update");
  const clamp = /* @__PURE__ */ __name((value) => {
    let nextValue = value;
    if (keepWithinRange) {
      nextValue = clampValue(nextValue, min, max);
    }
    return toPrecision(nextValue, precision);
  }, "clamp");
  const increment = /* @__PURE__ */ __name((step = stepProp) => {
    let next;
    if (valueState.value === "") {
      next = parse(step);
    } else {
      next = parse(valueState.value) + step;
    }
    next = clamp(next);
    update(next);
  }, "increment");
  const decrement = /* @__PURE__ */ __name((step = stepProp) => {
    let next;
    if (valueState.value === "") {
      next = parse(-step);
    } else {
      next = parse(valueState.value) - step;
    }
    next = clamp(next);
    update(next);
  }, "decrement");
  const isOutOfRange = ref(false);
  const valueAsNumber = ref(0);
  watchEffect(() => {
    decimalPlaces.value = getDecimalPlaces(parse(valueState.value), stepProp);
    precision = precisionProp != null ? precisionProp : decimalPlaces.value;
    valueAsNumber.value = parse(valueState.value);
    isOutOfRange.value = valueAsNumber.value > max || valueAsNumber.value < min;
  });
  return {
    decrement,
    increment,
    update,
    valueAsNumber,
    isOutOfRange,
    value: valueState
  };
}
__name(useCounter, "useCounter");
function parse(value) {
  return parseFloat(value.toString().replace(/[^\w.-]+/g, ""));
}
__name(parse, "parse");
function getDecimalPlaces(value, step) {
  return Math.max(countDecimalPlaces(step), countDecimalPlaces(value));
}
__name(getDecimalPlaces, "getDecimalPlaces");
function cast(value, step, precision) {
  const parsedValue = parse(value);
  if (Number.isNaN(parsedValue))
    return void 0;
  const decimalPlaces = getDecimalPlaces(parsedValue, step);
  return toPrecision(parsedValue, precision != null ? precision : decimalPlaces);
}
__name(cast, "cast");

export {
  useCounter
};
