"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};

// src/index.tsx
var src_exports = {};
__export(src_exports, {
  StackMessage: () => StackMessage,
  createDescendantRegister: () => createDescendantRegister,
  useClipboard: () => useClipboard,
  useCounter: () => useCounter,
  useDisclosure: () => useDisclosure,
  useElementStack: () => useElementStack,
  useEventListener: () => useEventListener,
  useId: () => useId,
  useIds: () => useIds,
  useMediaQuery: () => useMediaQuery,
  useStackContext: () => useStackContext,
  useStackProvider: () => useStackProvider,
  useSupported: () => useSupported,
  useWindowEvent: () => useWindowEvent
});
module.exports = __toCommonJS(src_exports);

// src/use-event-listener.ts
var import_vue = require("vue");
var import_utils = require("@chakra-ui/utils");
var import_vue_utils = require("@chakra-ui/vue-utils");
function useEventListener(...args) {
  let target;
  let event;
  let listener;
  let options;
  if ((0, import_utils.isString)(args[0])) {
    ;
    [event, listener, options] = args;
    target = import_vue_utils.defaultWindow;
  } else {
    ;
    [target, event, listener, options] = args;
  }
  if (!target)
    return import_utils.noop;
  let cleanup = import_utils.noop;
  const stopWatch = (0, import_vue.watch)(() => (0, import_vue.unref)(target), (el) => {
    cleanup();
    if (!el)
      return;
    el.addEventListener(event, listener, options);
    cleanup = /* @__PURE__ */ __name(() => {
      el.removeEventListener(event, listener, options);
      cleanup = import_utils.noop;
    }, "cleanup");
  }, {
    immediate: true,
    flush: "post"
  });
  const stop = /* @__PURE__ */ __name(() => {
    stopWatch();
    cleanup();
  }, "stop");
  (0, import_vue_utils.tryOnScopeDispose)(stop);
  return stop;
}
__name(useEventListener, "useEventListener");

// src/use-element-stack.ts
var import_vue2 = require("vue");
var StackContext = Symbol("ChakraElementStackContext");
var StackMessage = /* @__PURE__ */ ((StackMessage2) => {
  StackMessage2[StackMessage2["AddElement"] = 0] = "AddElement";
  StackMessage2[StackMessage2["RemoveElement"] = 1] = "RemoveElement";
  return StackMessage2;
})(StackMessage || {});
function useStackContext() {
  return (0, import_vue2.inject)(StackContext, () => {
  });
}
__name(useStackContext, "useStackContext");
function useElementStack(element) {
  let notify = useStackContext();
  (0, import_vue2.watchEffect)((onInvalidate) => {
    let domElement = element == null ? void 0 : element.value;
    if (!domElement)
      return;
    notify(0 /* AddElement */, domElement);
    onInvalidate(() => notify(1 /* RemoveElement */, domElement));
  });
}
__name(useElementStack, "useElementStack");
function useStackProvider(onUpdate) {
  let parentUpdate = useStackContext();
  function notify(...args) {
    onUpdate == null ? void 0 : onUpdate(...args);
    parentUpdate(...args);
  }
  __name(notify, "notify");
  (0, import_vue2.provide)(StackContext, notify);
}
__name(useStackProvider, "useStackProvider");

// src/use-clipboard.ts
var import_vue3 = require("vue");
function useClipboard({
  source,
  copyTimeout = 1e3,
  read = false
}) {
  const isSupported = Boolean(navigator && "clipboard" in navigator);
  const text = (0, import_vue3.ref)("");
  const copied = (0, import_vue3.ref)(false);
  const timeout = /* @__PURE__ */ __name(() => {
    let t = setTimeout(() => {
      copied.value = false;
    }, copyTimeout);
  }, "timeout");
  const events = ["copy", "cut"];
  if (isSupported && read) {
    for (const event of events)
      useEventListener(event, updateText);
  }
  function updateText() {
    navigator.clipboard.readText().then((value) => {
      text.value = value;
    });
  }
  __name(updateText, "updateText");
  async function copy(_value = (0, import_vue3.unref)(source)) {
    const value = (0, import_vue3.unref)(_value);
    if (isSupported && value != null) {
      await navigator.clipboard.writeText(value);
      text.value = value;
      copied.value = true;
      timeout();
    }
  }
  __name(copy, "copy");
  return {
    copy,
    text,
    copied
  };
}
__name(useClipboard, "useClipboard");

// src/use-counter.ts
var import_utils2 = require("@chakra-ui/utils");
var import_vue4 = require("vue");
function useCounter(props = {}) {
  const {
    precision: precisionProp,
    defaultValue = 0,
    step: stepProp = 1,
    min = Number.MIN_SAFE_INTEGER,
    max = Number.MAX_SAFE_INTEGER,
    keepWithinRange = true
  } = props;
  const valueState = (0, import_vue4.ref)((() => {
    var _a;
    if (defaultValue == null)
      return "";
    return (_a = cast(defaultValue, stepProp, precisionProp)) != null ? _a : "";
  })());
  const decimalPlaces = (0, import_vue4.ref)(0);
  let precision;
  const update = /* @__PURE__ */ __name((next) => {
    if (next === valueState.value)
      return;
    valueState.value = (0, import_utils2.toPrecision)(parse(next), precision).toString();
  }, "update");
  const clamp = /* @__PURE__ */ __name((value) => {
    let nextValue = value;
    if (keepWithinRange) {
      nextValue = (0, import_utils2.clampValue)(nextValue, min, max);
    }
    return (0, import_utils2.toPrecision)(nextValue, precision);
  }, "clamp");
  const increment = /* @__PURE__ */ __name((step = stepProp) => {
    let next;
    if (valueState.value === "") {
      next = parse(step);
    } else {
      next = parse(valueState.value) + step;
    }
    next = clamp(next);
    update(next);
  }, "increment");
  const decrement = /* @__PURE__ */ __name((step = stepProp) => {
    let next;
    if (valueState.value === "") {
      next = parse(-step);
    } else {
      next = parse(valueState.value) - step;
    }
    next = clamp(next);
    update(next);
  }, "decrement");
  const isOutOfRange = (0, import_vue4.ref)(false);
  const valueAsNumber = (0, import_vue4.ref)(0);
  (0, import_vue4.watchEffect)(() => {
    decimalPlaces.value = getDecimalPlaces(parse(valueState.value), stepProp);
    precision = precisionProp != null ? precisionProp : decimalPlaces.value;
    valueAsNumber.value = parse(valueState.value);
    isOutOfRange.value = valueAsNumber.value > max || valueAsNumber.value < min;
  });
  return {
    decrement,
    increment,
    update,
    valueAsNumber,
    isOutOfRange,
    value: valueState
  };
}
__name(useCounter, "useCounter");
function parse(value) {
  return parseFloat(value.toString().replace(/[^\w.-]+/g, ""));
}
__name(parse, "parse");
function getDecimalPlaces(value, step) {
  return Math.max((0, import_utils2.countDecimalPlaces)(step), (0, import_utils2.countDecimalPlaces)(value));
}
__name(getDecimalPlaces, "getDecimalPlaces");
function cast(value, step, precision) {
  const parsedValue = parse(value);
  if (Number.isNaN(parsedValue))
    return void 0;
  const decimalPlaces = getDecimalPlaces(parsedValue, step);
  return (0, import_utils2.toPrecision)(parsedValue, precision != null ? precision : decimalPlaces);
}
__name(cast, "cast");

// src/use-disclosure.ts
var import_vue6 = require("vue");

// src/use-id.ts
var import_utils3 = require("@chakra-ui/utils");
var import_vue5 = require("vue");
var serverHandoffComplete = (0, import_utils3.canUseDOM)();
var _id = 0;
var genId = /* @__PURE__ */ __name(() => ++_id, "genId");
var useId = /* @__PURE__ */ __name((id, prefix) => {
  const initialId = id || (serverHandoffComplete ? genId() : null);
  const uid = (0, import_vue5.ref)(initialId);
  (0, import_vue5.onBeforeMount)(() => {
    if (serverHandoffComplete === false) {
      serverHandoffComplete = true;
    }
  });
  (0, import_vue5.onMounted)(() => {
    if (uid.value === null) {
      uid.value = genId();
    }
  });
  return (0, import_vue5.computed)(() => {
    const __id__ = uid.value !== null ? uid.value.toString() : void 0;
    return prefix ? `${prefix}-${__id__}` : __id__;
  });
}, "useId");
function useIds(id, ...prefixes) {
  const __id__ = useId(id);
  return prefixes.map((prefix) => (0, import_vue5.computed)(() => `${prefix}-${__id__.value}`));
}
__name(useIds, "useIds");

// src/use-disclosure.ts
function useDisclosure(props = {}) {
  const {
    isOpen: isOpenProp,
    onClose: handleClose,
    onOpen: handleOpen,
    id: idProp,
    defaultIsOpen
  } = props;
  const isOpenState = (0, import_vue6.ref)(defaultIsOpen || false);
  const isOpen = (0, import_vue6.ref)(isOpenProp !== void 0 ? isOpenProp : isOpenState.value);
  const isControlled = isOpenProp !== void 0;
  const uid = useId();
  const id = (0, import_vue6.computed)(() => idProp != null ? idProp : `disclosure-${uid.value}`);
  const close = /* @__PURE__ */ __name(() => {
    if (!isControlled) {
      isOpenState.value = false;
    }
    handleClose == null ? void 0 : handleClose();
  }, "close");
  const open = /* @__PURE__ */ __name(() => {
    if (!isControlled) {
      isOpenState.value = true;
    }
    handleOpen == null ? void 0 : handleOpen();
  }, "open");
  const toggle = /* @__PURE__ */ __name(() => isOpen.value ? close() : open(), "toggle");
  const buttonProps = (0, import_vue6.computed)(() => ({
    "aria-expanded": isOpen.value,
    "aria-controls": id.value,
    onClick() {
      toggle();
    }
  }));
  const disclosureProps = (0, import_vue6.computed)(() => ({
    hidden: !isOpen.value,
    id: id.value
  }));
  (0, import_vue6.watchEffect)(() => {
    isOpen.value = isOpenState.value;
  });
  return {
    isOpen,
    open,
    close,
    toggle,
    isControlled,
    buttonProps,
    disclosureProps
  };
}
__name(useDisclosure, "useDisclosure");

// src/use-media-query.ts
var import_vue_utils3 = require("@chakra-ui/vue-utils");

// src/use-supported.ts
var import_vue_utils2 = require("@chakra-ui/vue-utils");
var import_vue7 = require("vue");
function useSupported(callback, sync = false) {
  const isSupported = (0, import_vue7.ref)();
  const update = /* @__PURE__ */ __name(() => isSupported.value = Boolean(callback()), "update");
  update();
  (0, import_vue_utils2.tryOnMounted)(update, sync);
  return isSupported;
}
__name(useSupported, "useSupported");

// src/use-media-query.ts
var import_vue8 = require("vue");
function useMediaQuery(query, options = {}) {
  const {
    window: window2 = import_vue_utils3.defaultWindow
  } = options;
  const isSupported = useSupported(() => window2 && "matchMedia" in window2 && typeof window2.matchMedia === "function");
  let mediaQuery;
  const matches = (0, import_vue8.ref)(false);
  const cleanup = /* @__PURE__ */ __name(() => {
    if (!mediaQuery)
      return;
    if ("removeEventListener" in mediaQuery)
      mediaQuery.removeEventListener("change", update);
    else
      mediaQuery.removeListener(update);
  }, "cleanup");
  const update = /* @__PURE__ */ __name(() => {
    if (!isSupported.value)
      return;
    cleanup();
    mediaQuery = window2.matchMedia((0, import_vue_utils3.resolveRef)(query).value);
    matches.value = mediaQuery.matches;
    if ("addEventListener" in mediaQuery)
      mediaQuery.addEventListener("change", update);
    else
      mediaQuery.addListener(update);
  }, "update");
  (0, import_vue8.watchEffect)(update);
  (0, import_vue_utils3.tryOnScopeDispose)(() => cleanup());
  return matches;
}
__name(useMediaQuery, "useMediaQuery");

// src/use-window-event.ts
var import_vue9 = require("vue");
function useWindowEvent(type, listener, options) {
  if (typeof window === "undefined")
    return;
  (0, import_vue9.watchEffect)((onInvalidate) => {
    window.addEventListener(type, listener, options);
    onInvalidate(() => {
      window.removeEventListener(type, listener, options);
    });
  });
}
__name(useWindowEvent, "useWindowEvent");

// src/use-descendant.ts
var import_vue_utils4 = require("@chakra-ui/vue-utils");
var import_vue10 = require("vue");
var DescendantAuthority = class {
  constructor(id) {
    __publicField(this, "descendants");
    __publicField(this, "id");
    this.descendants = [];
    this.id = id;
  }
  register(descendant) {
    this.descendants.push(descendant);
    return descendant.id;
  }
  unregister(id) {
    this.descendants = this.descendants.filter((descendant) => descendant.id !== id);
  }
  isRegistered(id) {
    return this.descendants.some((descendant) => descendant.id === id);
  }
  getDescendants() {
    return this.descendants;
  }
  getDescendantById(id) {
    return this.descendants.find((descendant) => descendant.id === id);
  }
  json() {
    const payload = {
      id: this.id,
      descendants: this.descendants.map((descendant) => ({
        id: descendant.id,
        authorityId: descendant.authorityId
      }))
    };
    return payload;
  }
};
__name(DescendantAuthority, "DescendantAuthority");
var descendantManagerStore = /* @__PURE__ */ new Map();
var Descendant = class {
  constructor({
    id,
    authority
  }) {
    __publicField(this, "id");
    __publicField(this, "authorityId");
    this.id = id;
    this.authorityId = authority.id;
  }
};
__name(Descendant, "Descendant");
function createDescendantAuthority(id) {
  const descendantManager = new DescendantAuthority(id);
  descendantManagerStore.set(id, descendantManager);
  return descendantManager;
}
__name(createDescendantAuthority, "createDescendantAuthority");
function createDescendant(id, authority) {
  return new Descendant({
    id,
    authority
  });
}
__name(createDescendant, "createDescendant");
function storeDescendant(descendant) {
  const {
    id,
    authorityId
  } = descendant;
  const authority = descendantManagerStore.get(authorityId);
  if (authority.isRegistered(id)) {
    throw new Error(`Descendant with id ${id} is already registered.`);
  }
  return authority.register(descendant);
}
__name(storeDescendant, "storeDescendant");
function createDescendantRegister(id) {
  let __descendantAuthority__ = createDescendantAuthority(Symbol(id));
  const descendantAuthority = (0, import_vue10.customRef)((track, trigger) => ({
    get() {
      track();
      return __descendantAuthority__;
    },
    set(value) {
      __descendantAuthority__ = value;
      trigger();
    }
  }));
  const [DescendantAuthorityProvider, useAuthority] = (0, import_vue_utils4.createContext)({
    name: id,
    strict: false
  }, __descendantAuthority__);
  function registerAuthority() {
    DescendantAuthorityProvider(__descendantAuthority__);
  }
  __name(registerAuthority, "registerAuthority");
  function registerDescendant(id2) {
    const descendant = createDescendant(id2, __descendantAuthority__);
    storeDescendant(descendant);
    (0, import_vue10.triggerRef)(descendantAuthority);
    return descendant;
  }
  __name(registerDescendant, "registerDescendant");
  return [registerAuthority, registerDescendant, useAuthority, descendantAuthority];
}
__name(createDescendantRegister, "createDescendantRegister");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  StackMessage,
  createDescendantRegister,
  useClipboard,
  useCounter,
  useDisclosure,
  useElementStack,
  useEventListener,
  useId,
  useIds,
  useMediaQuery,
  useStackContext,
  useStackProvider,
  useSupported,
  useWindowEvent
});
