"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/use-counter.ts
var use_counter_exports = {};
__export(use_counter_exports, {
  useCounter: () => useCounter
});
module.exports = __toCommonJS(use_counter_exports);
var import_utils = require("@chakra-ui/utils");
var import_vue = require("vue");
function useCounter(props = {}) {
  const {
    precision: precisionProp,
    defaultValue = 0,
    step: stepProp = 1,
    min = Number.MIN_SAFE_INTEGER,
    max = Number.MAX_SAFE_INTEGER,
    keepWithinRange = true
  } = props;
  const valueState = (0, import_vue.ref)((() => {
    var _a;
    if (defaultValue == null)
      return "";
    return (_a = cast(defaultValue, stepProp, precisionProp)) != null ? _a : "";
  })());
  const decimalPlaces = (0, import_vue.ref)(0);
  let precision;
  const update = /* @__PURE__ */ __name((next) => {
    if (next === valueState.value)
      return;
    valueState.value = (0, import_utils.toPrecision)(parse(next), precision).toString();
  }, "update");
  const clamp = /* @__PURE__ */ __name((value) => {
    let nextValue = value;
    if (keepWithinRange) {
      nextValue = (0, import_utils.clampValue)(nextValue, min, max);
    }
    return (0, import_utils.toPrecision)(nextValue, precision);
  }, "clamp");
  const increment = /* @__PURE__ */ __name((step = stepProp) => {
    let next;
    if (valueState.value === "") {
      next = parse(step);
    } else {
      next = parse(valueState.value) + step;
    }
    next = clamp(next);
    update(next);
  }, "increment");
  const decrement = /* @__PURE__ */ __name((step = stepProp) => {
    let next;
    if (valueState.value === "") {
      next = parse(-step);
    } else {
      next = parse(valueState.value) - step;
    }
    next = clamp(next);
    update(next);
  }, "decrement");
  const isOutOfRange = (0, import_vue.ref)(false);
  const valueAsNumber = (0, import_vue.ref)(0);
  (0, import_vue.watchEffect)(() => {
    decimalPlaces.value = getDecimalPlaces(parse(valueState.value), stepProp);
    precision = precisionProp != null ? precisionProp : decimalPlaces.value;
    valueAsNumber.value = parse(valueState.value);
    isOutOfRange.value = valueAsNumber.value > max || valueAsNumber.value < min;
  });
  return {
    decrement,
    increment,
    update,
    valueAsNumber,
    isOutOfRange,
    value: valueState
  };
}
__name(useCounter, "useCounter");
function parse(value) {
  return parseFloat(value.toString().replace(/[^\w.-]+/g, ""));
}
__name(parse, "parse");
function getDecimalPlaces(value, step) {
  return Math.max((0, import_utils.countDecimalPlaces)(step), (0, import_utils.countDecimalPlaces)(value));
}
__name(getDecimalPlaces, "getDecimalPlaces");
function cast(value, step, precision) {
  const parsedValue = parse(value);
  if (Number.isNaN(parsedValue))
    return void 0;
  const decimalPlaces = getDecimalPlaces(parsedValue, step);
  return (0, import_utils.toPrecision)(parsedValue, precision != null ? precision : decimalPlaces);
}
__name(cast, "cast");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useCounter
});
