import * as vue from 'vue';

/**
 * The purpose of this useDescendant is to provide a way to register a descendant
 * component and provide it with a unique index. This is useful for things like
 * accordions, tabs, or any other type of list where only one item can be active
 * at a time.
 */
declare class DescendantAuthority {
    constructor(id: symbol);
    descendants: Descendant[];
    id: symbol;
    register(descendant: Descendant): string;
    unregister(id: string): void;
    isRegistered(id: string): boolean;
    getDescendants(): Descendant[];
    getDescendantById(id: string): Descendant | undefined;
    json(): {
        id: symbol;
        descendants: {
            id: string;
            authorityId: symbol;
        }[];
    };
}
declare class Descendant {
    constructor({ id, authority, }: {
        id: string;
        authority: DescendantAuthority;
    });
    id: string;
    authorityId: symbol;
}
declare function createDescendantRegister(id: string): readonly [() => void, (id: string) => Descendant, (fallback?: DescendantAuthority | undefined) => DescendantAuthority, vue.Ref<DescendantAuthority>];

export { Descendant, DescendantAuthority, createDescendantRegister };
