"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};

// src/use-descendant.ts
var use_descendant_exports = {};
__export(use_descendant_exports, {
  Descendant: () => Descendant,
  DescendantAuthority: () => DescendantAuthority,
  createDescendantRegister: () => createDescendantRegister
});
module.exports = __toCommonJS(use_descendant_exports);
var import_vue_utils = require("@chakra-ui/vue-utils");
var import_vue = require("vue");
var DescendantAuthority = class {
  constructor(id) {
    __publicField(this, "descendants");
    __publicField(this, "id");
    this.descendants = [];
    this.id = id;
  }
  register(descendant) {
    this.descendants.push(descendant);
    return descendant.id;
  }
  unregister(id) {
    this.descendants = this.descendants.filter((descendant) => descendant.id !== id);
  }
  isRegistered(id) {
    return this.descendants.some((descendant) => descendant.id === id);
  }
  getDescendants() {
    return this.descendants;
  }
  getDescendantById(id) {
    return this.descendants.find((descendant) => descendant.id === id);
  }
  json() {
    const payload = {
      id: this.id,
      descendants: this.descendants.map((descendant) => ({
        id: descendant.id,
        authorityId: descendant.authorityId
      }))
    };
    return payload;
  }
};
__name(DescendantAuthority, "DescendantAuthority");
var descendantManagerStore = /* @__PURE__ */ new Map();
var Descendant = class {
  constructor({
    id,
    authority
  }) {
    __publicField(this, "id");
    __publicField(this, "authorityId");
    this.id = id;
    this.authorityId = authority.id;
  }
};
__name(Descendant, "Descendant");
function createDescendantAuthority(id) {
  const descendantManager = new DescendantAuthority(id);
  descendantManagerStore.set(id, descendantManager);
  return descendantManager;
}
__name(createDescendantAuthority, "createDescendantAuthority");
function createDescendant(id, authority) {
  return new Descendant({
    id,
    authority
  });
}
__name(createDescendant, "createDescendant");
function storeDescendant(descendant) {
  const {
    id,
    authorityId
  } = descendant;
  const authority = descendantManagerStore.get(authorityId);
  if (authority.isRegistered(id)) {
    throw new Error(`Descendant with id ${id} is already registered.`);
  }
  return authority.register(descendant);
}
__name(storeDescendant, "storeDescendant");
function createDescendantRegister(id) {
  let __descendantAuthority__ = createDescendantAuthority(Symbol(id));
  const descendantAuthority = (0, import_vue.customRef)((track, trigger) => ({
    get() {
      track();
      return __descendantAuthority__;
    },
    set(value) {
      __descendantAuthority__ = value;
      trigger();
    }
  }));
  const [DescendantAuthorityProvider, useAuthority] = (0, import_vue_utils.createContext)({
    name: id,
    strict: false
  }, __descendantAuthority__);
  function registerAuthority() {
    DescendantAuthorityProvider(__descendantAuthority__);
  }
  __name(registerAuthority, "registerAuthority");
  function registerDescendant(id2) {
    const descendant = createDescendant(id2, __descendantAuthority__);
    storeDescendant(descendant);
    (0, import_vue.triggerRef)(descendantAuthority);
    return descendant;
  }
  __name(registerDescendant, "registerDescendant");
  return [registerAuthority, registerDescendant, useAuthority, descendantAuthority];
}
__name(createDescendantRegister, "createDescendantRegister");
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Descendant,
  DescendantAuthority,
  createDescendantRegister
});
