import { Ref } from 'vue';

/**
 * SSR compatible way to check if something is supported. If it is, it returns true, otherwise false.
 *
 * ```ts
 *  import { useSupported } from '@vueuse/core'
 *
 *  const isSupported = useSupported(() => navigator && 'getBattery' in navigator)
 *
 *  if (isSupported.value) {
 *    // do something
 *    navigator.getBattery
 *  }
 *  ```
 *
 * @param callback
 * @param sync
 * @returns
 */
declare function useSupported(callback: () => unknown, sync?: boolean): Ref<boolean>;

export { useSupported };
